;==============================================================================
; Control Logic
;------------------------------------------------------------------------------

; Notes:
;
; This module contains the main control logic for the metronome including
; all the button press and EEPROM access code.

;==============================================================================
; Revision History:
;
; 2006-01-29 AJ Initial version
; 2006-03-09 AJ Debugged and commented.
;------------------------------------------------------------------------------

                errorlevel -302
                errorlevel -312

                include ../pic.inc
                include ../banks.inc
                include Hardware.inc
                include ../hex-patterns.inc

                global  SPKR

                global  ControlLogic

                extern  DIGIT
                extern  SEGMENTS
                extern  T1_VALUE
                extern  DIVISOR
                extern  TICKS

                extern  GetBpmCount
                extern  GetBpmValue
                extern  GetTimerLo
                extern  GetTimerHi
                extern  EepromSetAddr
                extern  EepromRead
                extern  EepromWriteWait

;==============================================================================

MSK_BLANK       equ     h'ff'^(1<<PWM_SPKR)

SETTINGS        equ     0               ; EEPROM index for settings byte

;==============================================================================

                udata

SAVED           res     1               ; Saved configuration
BPM             res     1               ; BPM index value (0..n-1)
SPKR            res     1               ; Speaker state (in bit 7)

BCD_H           res     1               ; BPM value broken down into hundreds
BCD_T           res     1               ; .. tens and
BCD_U           res     1               ; .. units

SCRATCH         res     1               ; General scratch register

;==============================================================================
; Initialisation
;------------------------------------------------------------------------------

                code

ControlLogic:
                BANKSYM SPKR
                clrf    SPKR            ; Clear the speaker flag

                movlw   SETTINGS        ; Recover last settings from EEPROM
                lcall   EepromSetAddr
                lcall   EepromRead

                BANKSYM SAVED
                movwf   SAVED           ; Save a copy
                andlw   h'80'           ; Extract speaker ON/OFF bit
                btfss   STATUS,Z
                decf    SPKR,F          ; Force speaker to ON

                movf    SAVED,W         ; Extract saved index
                andlw   h'7f'
                movwf   BPM             ; And save

                lcall   GetBpmCount     ; Get maximum value
                subwf   BPM,W           ; And compare
                btfsc   STATUS,C        ; Skip reset if in range
                clrf    BPM             ; Otherwise reset to zero

                lcall   BpmChanged      ; Show the initial rate
                pagesel $

                movlw   h'ff'

                BANKSYM DIVISOR         ; Initialise postscale divisor
                movwf   DIVISOR                

                BANKSYM DIGIT
                movwf   DIGIT           ; Initialise DIGIT counter

                BANKABS TMR0
                movwf   TMR0            ; Set timer0 to roll over quickly 

                BANKABS TMR1L           ; Set timer1 to roll over quickly
                movwf   TMR1L
                movwf   TMR1H

                BANKABS PIE1
                bsf     PIE1,TMR1IE     ; Enable timer interrupts
                bsf     INTCON,T0IE
                bsf     INTCON,PEIE
                bsf     INTCON,GIE      ; And start interrupt processing

;==============================================================================
; Main Processing Loop
;------------------------------------------------------------------------------

EventLoop:
                BANKANY
                clrwdt                  ; Prevent reset

;------------------------------------------------------------------------------
                
CheckDnButton:
                BANKABS PORTA           
                btfsc   PORTA,SW_DN     ; Is the BPM down button pressed?
                goto    CheckUpButton   ; No, check the next button


                BANKSYM TICKS
                movlw   -DEBOUNCE
                movwf   TICKS
WaitDn:
                clrwdt                  ; Prevent reset
                movf    TICKS,W         ; Has repeat counter reached zero?
                btfss   STATUS,Z
                BANKABS PORTA 
                btfsc   PORTA,SW_DN     ; Is DN button still pressed?
                goto    NextIteration   ; No, must have been a bounce
SlowDn:
                call    BumpBpmDn       ; Decrease the BPM index
                call    BpmChanged      ; .. and redisplay

                BANKSYM TICKS
                movlw   -SLOW           ; Set repeat counter for SLOW
                movwf   TICKS
WaitSlowDn:
                clrwdt                  ; Prevent reset
                movf    TICKS,W         ; Has repeat counter reached zero?
                btfss   STATUS,Z
                goto    WaitSlowDn      ; Not yet.
                BANKABS PORTA
                btfsc   PORTA,SW_DN     ; Has the button been released?
                goto    NextIteration   ; Yes, update stored value
                goto    SlowDn          ; No, repeat DN operation

;------------------------------------------------------------------------------

CheckUpButton:
                btfsc   PORTA,SW_UP     ; Is the BPM up button pressed?
                goto    CheckSpButton   ; No, check the next button

                BANKSYM TICKS
                movlw   -DEBOUNCE
                movwf   TICKS
WaitUp:
                clrwdt                  ; Prevent reset
                movf    TICKS,W         ; Has repeat counter reached zero?
                btfss   STATUS,Z
                BANKABS PORTA 
                btfsc   PORTA,SW_UP     ; Is UP button still pressed?
                goto    NextIteration   ; No, must have been a bounce
SlowUp:
                call    BumpBpmUp       ; Increase the BPM index
                call    BpmChanged      ; .. and redisplay

                BANKSYM TICKS
                movlw   -SLOW           ; Set repeat counter for SLOW
                movwf   TICKS
WaitSlowUp:
                clrwdt                  ; Prevent reset
                movf    TICKS,W         ; Has repeat counter reached zero?
                btfss   STATUS,Z
                goto    WaitSlowUp      ; Not yet.
                BANKABS PORTA
                btfsc   PORTA,SW_UP     ; Has the button been released?
                goto    NextIteration   ; Yes, update stored value
                goto    SlowUp          ; No, repeat UP operation

;------------------------------------------------------------------------------

CheckSpButton:
                btfsc   PORTA,SW_SPKR   ; Is speaker toggle button pressed
                goto    NextIteration

                BANKSYM TICKS
                movlw   -DEBOUNCE       ; Set counter for debouncing
                movwf   TICKS
WaitSpkr:
                clrwdt                  ; Prevent reset
                movf    TICKS,W         ; Has debounce counter reached zero?
                btfss   STATUS,Z
                goto    WaitSpkr        ; Not yet.
                
                BANKABS PORTA 
                btfsc   PORTA,SW_SPKR   ; Is speaker button still pressed?
                goto    NextIteration   ; No, must have been a bounce

                BANKSYM SPKR
                btfsc   SPKR,7          ; Toggle speaker state
                goto    SpkrOff

                bsf     SPKR,7          ; Set speaker off
                call    ShowOn          ; Display message
                goto    WaitForRelease
SpkrOff:
                bcf     SPKR,7          ; Set speaker off
                call    ShowOff         ; Display message

WaitForRelease:
                clrwdt                  ; Prevent reset
                BANKABS PORTA 
                btfss   PORTA,SW_SPKR   ; Is speaker button still pressed?
                goto    WaitForRelease  ; Yes, wait some more

                call    ShowBpm

;------------------------------------------------------------------------------

NextIteration:
                movlw   SETTINGS        ; Prepare EEPROM for write
                lcall   EepromSetAddr
                pagesel $               ; Ensure page bits are correct

                BANKSYM SAVED
                movf    BPM,W           ; Fetch BPM index
                btfsc   SPKR,7          ; And merge in speaker bit
                iorlw   h'80'

                xorwf   SAVED,W         ; Same as last saved value?
                btfsc   STATUS,Z       
                goto    EventLoop       ; Yes,
                xorwf   SAVED,W         ; Recover new value

                movwf   SAVED           ; And update EEPROM
                lcall   EepromWriteWait
                lgoto   EventLoop

;==============================================================================

; Add one from the BPM index, wrapping it back round to zero if the we reach
; the maximum.
;
; Bank changed. Uses 2 stack levels.

BumpBpmUp:
                lcall   GetBpmCount     ; Fetch the number of BPM values
                pagesel $
                BANKSYM BPM
                incf    BPM,F           ; Increment by one
                xorwf   BPM,W           ; And compare with BPM count
                btfsc   STATUS,Z        ; Reset if wrapped round.
                clrf    BPM             
                return

; Subtract one form the BPM index, wrapping it back round to the maximum if
; it is zero.
;
; Bank changed. Uses 2 stack levels.

BumpBpmDn:
                lcall   GetBpmCount     ; Fetch the number of BPM values
                pagesel $
                BANKSYM BPM
                incf    BPM,F           ; Test if BPM is zero
                decf    BPM,F
                btfsc   STATUS,Z        ; Reset if wrapped round
                movwf   BPM
                decf    BPM,F           ; Do final decrement
                return

;==============================================================================

; The BPM index has been changed. Recompute the BCD value to display on the
; LEDs and set the timer value so the interrupt routine will switch to the
; new frequency.
;
; Bank changed. Uses 2 stack levels.

BpmChanged:
                BANKSYM BPM
                movlw   h'ff'           ; Prepare BCD conversion area
                movwf   BCD_H
                movwf   BCD_T

                movf    BPM,W           ; Get BPM value for current index
                lcall   GetBpmValue
                pagesel $               ; Ensure correct page settings

Remove100s:
                incf    BCD_H,F         ; Assume we can take out a 100
                addlw   -.100           ; Then try to do it
                btfsc   STATUS,C
                goto    Remove100s      ; Repeat until we failed
                addlw   .100            ; .. and repair the value
 
Remove10s:
                incf    BCD_T,F         ; Assume we can take out a 10
                addlw   -.10            ; Then try to do it
                btfsc   STATUS,C
                goto    Remove10s       ; Repeat until we failed
                addlw   .10             ; .. and repair the value

                movwf   BCD_U           ; The units must be left

                BANKSYM BPM
                movf    BPM,W           ; Recover BPM index
                lcall   GetTimerLo      ; Convert to timer lo byte
                BANKSYM T1_VALUE
                movwf   T1_VALUE+0
                BANKSYM BPM
                movf    BPM,W           ; Recover BPM index
                lcall   GetTimerHi      ; Convert to timer hi byte
                pagesel $               ; Ensure correct page settings
                BANKSYM T1_VALUE
                movwf   T1_VALUE+1

; Converts the decomposed BCD value into the bit patterns to control the
; seven segment LEDS and stores in the display pattern area.
;
; Bank changed. Uses 2 stack levels.

ShowBpm:
                BANKSYM BCD_H
                movf    BCD_H,W         ; Should 100s be blank?
                btfsc   STATUS,Z
                goto    Blank100s       ; Yes, just do 10s and 1s

Show100s:
                BANKSYM BCD_H
                movf    BCD_H,W         ; Fetch 100s digit
                call    GetSegments     ; And convert to segment pattern
Blank100s
                BANKSYM SEGMENTS
                xorlw   MSK_BLANK
                movwf   SEGMENTS+0      ; And save in display pattern

Show10s:
                BANKSYM BCD_T
                movf    BCD_T,W         ; Fetch 10s digit
                call    GetSegments     ; And convert to segment pattern
                BANKSYM SEGMENTS
                xorlw   MSK_BLANK
                movwf   SEGMENTS+1      ; And save in display pattern

                BANKSYM BCD_U
                movf    BCD_U,W         ; Fetch 1s digit
                call    GetSegments     ; And convert to segment pattern
                BANKSYM SEGMENTS
                xorlw   MSK_BLANK
                movwf   SEGMENTS+2      ; And save in display pattern
                return

; Sets the display pattern area so that the word 'On' will shown when the
; LEDs are refreshed
;
; WREG contains result. Bank changed. Uses 1 stack level.

ShowOn:
                BANKSYM SEGMENTS
                movlw   (1<<LED_A)|(1<<LED_B)|(1<<LED_C)|(1<<LED_D)|(1<<LED_E)|(1<<LED_F)
                xorlw   MSK_BLANK
                movwf   SEGMENTS+0
                movlw   (1<<LED_C)|(1<<LED_E)|(1<<LED_G)
                xorlw   MSK_BLANK
                movwf   SEGMENTS+1
                movlw   MSK_BLANK
                movwf   SEGMENTS+2
                return

; Sets the display pattern area so that the word 'Off' will shown when the
; LEDs are refreshed
;
; WREG contains result. Bank changed. Uses 1 stack level.

ShowOff:                
                BANKSYM SEGMENTS
                movlw   (1<<LED_A)|(1<<LED_B)|(1<<LED_C)|(1<<LED_D)|(1<<LED_E)|(1<<LED_F)
                xorlw   MSK_BLANK
                movwf   SEGMENTS+0
                movlw   (1<<LED_A)|(1<<LED_E)|(1<<LED_F)|(1<<LED_G)
                xorlw   MSK_BLANK
                movwf   SEGMENTS+1
                movwf   SEGMENTS+2
                return

; Uses a computed jump into a table to fetch the bit pattern corresponding to hex
; digit 0-f. Values (a-f) shown never be displayed.
;
; WREG contains result. Bank changed. Uses 1 stack level.

GetSegments:
                BANKSYM SCRATCH
                andlw   h'0f'
                movwf   SCRATCH
                movlw   high Patterns
                movwf   PCLATH
                movf    SCRATCH,W
                addlw   low Patterns
                btfsc   STATUS,C
                incf    PCLATH,F
                movwf   PCL
                
Patterns:
                HEX_PATTERNS    LED_A,LED_B,LED_C,LED_D,LED_E,LED_F,LED_G

		end
