;==============================================================================
; Data Tables
;------------------------------------------------------------------------------

; Notes:
;
; The metronome stores the current BPM as index (0..N-1) which is used to
; access a set of tables that contain the actual BPM value (in binary) and the
; the counter value loaded into TIMER 2.
;
; As timers increment the timer values are computed as 16-bit negative numbers. 
;
; All the tables must have EXACTLY the same number on entries or the address
; calculations will jump to wrong location. The total combined size of ALL the
; tables MUST be 256 bytes or less.
 
;==============================================================================
; Revision History:
;
; 2006-02-02 AJ Initial version
;------------------------------------------------------------------------------

                include ../pic.inc
                include ../banks.inc
                include Hardware.inc

                global  GetBpmCount
		global	GetBpmValue
		global	GetTimerHi
		global	GetTimerLo

;------------------------------------------------------------------------------

; LO and HI compute bytes of the timer based on the required BPM and current
; oscillator and pre/post scaler settings.

#define LO(BPM) ((-.60 * FOSC/PRESCALE)/(BPM * POSTSCALE)) >> 0 & h'ff'
#define HI(BPM) ((-.60 * FOSC/PRESCALE)/(BPM * POSTSCALE)) >> 8 & h'ff'

;==============================================================================

                udata

SCRATCH         res     1               ; Used during address calculation

;==============================================================================

		code

; Returns the number of entries in the BPM and timer tables based on thier
; current size.
;
; WREG contains result. No bank changes. Uses 1 stack level.

GetBpmCount:
                retlw   HiBytes-Bpms

; Returns the high byte of the value used to configure TIMER1 for the
; BPM index in WREG.
;
; WREG contains result. Bank changed. Uses 1 stack level.

GetTimerHi:
                addlw   HiBytes-Bpms    ; Adjust index to correct table 
                goto    GetBpmValue     ; .. before reading

; Returns the low byte of the value used to configure TIMER1 for the
; BPM index in WREG.
;
; WREG contains result. Bank changed. Uses 1 stack level.

GetTimerLo:
                addlw   LoBytes-Bpms    ; Adjust index to correct table

; Returns the BPM value of corresponding to the index in WREG.
;
; WREG contains result. Bank changed. Uses 1 stack level.

GetBpmValue:
                BANKSYM SCRATCH
                movwf   SCRATCH         ; Save index value for later         
                movlw   high Bpms       ; Set PCLATH ready for computed jump 
                movwf   PCLATH
                movf    SCRATCH,W       ; Recover index value
                addlw   low Bpms        ; Compute table entry point
                btfsc   STATUS,C        ; Have we gone over a page boundary?
                incf    PCLATH,F        ; Yes, correct high byte
                movwf   PCL             ; And jump into table

;------------------------------------------------------------------------------

; A table of BPM counts (in binary) 

Bpms:
                dt      .30,  .32
                dt      .34,  .36
                dt      .38,  .40
                dt      .42,  .44
                dt      .46,  .48
                dt      .50,  .52
                dt      .54,  .56
                dt      .58,  .60
                dt      .63,  .66
                dt      .69,  .72
                dt      .76,  .80
                dt      .84,  .88
                dt      .92,  .96
                dt      .100, .104
                dt      .108, .112
                dt      .116, .120
                dt      .126, .132
                dt      .138, .144
                dt      .152, .160
                dt      .168, .176
                dt      .184, .192
                dt      .200, .208

; HI byte of the 16-bit timer value.

HiBytes:
                dt      HI(.30),  HI(.32)
                dt      HI(.34),  HI(.36)
                dt      HI(.38),  HI(.40)
                dt      HI(.42),  HI(.44)
                dt      HI(.46),  HI(.48)
                dt      HI(.50),  HI(.52)
                dt      HI(.54),  HI(.56)
                dt      HI(.58),  HI(.60)
                dt      HI(.63),  HI(.66)
                dt      HI(.69),  HI(.72)
                dt      HI(.76),  HI(.80)
                dt      HI(.84),  HI(.88)
                dt      HI(.92),  HI(.96)
                dt      HI(.100), HI(.104)
                dt      HI(.108), HI(.112)
                dt      HI(.116), HI(.120)
                dt      HI(.126), HI(.132)
                dt      HI(.138), HI(.144)
                dt      HI(.152), HI(.160)
                dt      HI(.168), HI(.176)
                dt      HI(.184), HI(.192)
                dt      HI(.200), HI(.208)

; LO byte of the 16-bit timer value.

LoBytes:
                dt      LO(.30),  LO(.32)
                dt      LO(.34),  LO(.36)
                dt      LO(.38),  LO(.40)
                dt      LO(.42),  LO(.44)
                dt      LO(.46),  LO(.48)
                dt      LO(.50),  LO(.52)
                dt      LO(.54),  LO(.56)
                dt      LO(.58),  LO(.60)
                dt      LO(.63),  LO(.66)
                dt      LO(.69),  LO(.72)
                dt      LO(.76),  LO(.80)
                dt      LO(.84),  LO(.88)
                dt      LO(.92),  LO(.96)
                dt      LO(.100), LO(.104)
                dt      LO(.108), LO(.112)
                dt      LO(.116), LO(.120)
                dt      LO(.126), LO(.132)
                dt      LO(.138), LO(.144)
                dt      LO(.152), LO(.160)
                dt      LO(.168), LO(.176)
                dt      LO(.184), LO(.192)
                dt      LO(.200), LO(.208)

		end
