;==============================================================================
; Interrupt Handler
;------------------------------------------------------------------------------

; Notes:
;
; A custom linker file is used to allow the interrupt routine to begin at
; h'004'.
;
; The interrupt routine uses one stack level during its processing.

;==============================================================================
; Revision History:
;
; 2006-01-30 AJ Initial version
;------------------------------------------------------------------------------

                errorlevel -302
                errorlevel -312

                include ../pic.inc
                include ../banks.inc
                include ../jump-table.inc

                include hardware.inc

                extern  _WREG
                extern  SPKR

                global  DIGIT
                global  SEGMENTS
                global  TICKS
                global  T1_VALUE
                global  DIVISOR

;==============================================================================

; Constant value used to set the timer interrupt interval

TMR0_INC        equ     -.26            ; Timer0 increment
 
;==============================================================================

; Critical registers are stored in shared ram, so that can be accessed from
; any bank.

                udata_shr

_STATUS         res     1               ; Saved STATUS register
_FSR            res     1               ; Saved FSR register
_PCLATH         res     1               ; Saved PCLATH register

; Main interrupt processing variables are stored in banked registers.

                udata

DIGIT           res     1               ; The digit being displayed (0-2)
SEGMENTS        res     3               ; Bit patterns for active segments

T1_VALUE        res     2               ; TIMER 1 reset value 
DIVISOR         res     1               ; Post scale divisor

TICKS           res     1               ; General Tick counter
PWM_TICKS       res     1               ; PWM tick counter

SCRATCH         res     1               ; Interrupt scratch area


;==============================================================================
; Interrupt Handler
;------------------------------------------------------------------------------

.interrupt      code    h'004'

Interrupt:
                movwf   _WREG           ; Save WREG
                movf    STATUS,W        ; Save a copy of current STATUS
                movwf   _STATUS
                movf    FSR,W           ; Save a copy of current FSR
                movwf   _FSR
                movf    PCLATH,W        ; Save a copy of current PCLATH
                movwf   _PCLATH
                pagesel $               ; Ensure correct page is selected

;-----------------------------------------------------------------------------

CheckTimer1:
                BANKSYM PIR1
                btfss   PIR1,TMR1IF     ; Did TIMER 1 cause the interrupt
                goto    CheckTimer0     ; Nope.
                bcf     PIR1,TMR1IF     ; Yes, clear the interrupt flag

                movlw   T1_VALUE        ; Point FSR at timer initial value
                BANKIND T1_VALUE
                movwf   FSR

                BANKABS T1CON
                bcf     T1CON,TMR1ON    ; Disable timer during reset
                movf    INDF,W          ; Add timer lo byte
                addwf   TMR1L,F
                btfsc   STATUS,C
                incf    TMR1H,F
                incf    FSR,F
                movf    INDF,W          ; Add timer hi byte
                addwf   TMR1H,F
                bsf     T1CON,TMR1ON    ; And reenable timer

                BANKSYM DIVISOR
                incfsz  DIVISOR,F       ; Increment divisor
                goto    CheckTimer0

                movlw   -POSTSCALE      ; Reset divisor
                movwf   DIVISOR

                movlw   -PULSE_TICKS    ; Set up PWM on timer
                movwf   PWM_TICKS

                BANKABS PORTA
                bsf     PORTA,LED_PULSE ; Turn the LED on
                BANKSYM SPKR
                btfss   SPKR,7          ; Is the speaker turned on?
                goto    CheckTimer0     ; No.

                BANKABS CCP1CON
                movlw   (1<<CCP1M3)|(1<<CCP1M2)
                movwf   CCP1CON         ; Configure PWM output
                movlw   .70
                movwf   CCPR1L
                BANKABS PR2
                movlw   .141
                movwf   PR2
                BANKABS T2CON
                bsf     T2CON,TMR2ON    ; Enable timer and generate PWM

;------------------------------------------------------------------------------

CheckTimer0:
                btfss   INTCON,T0IF     ; Did TIMER 0 cause the interrupt?
                goto    Finished        ; Nope.
                bcf     INTCON,T0IF     ; Yes, clear the interrupt flag

                BANKABS TMR0
                movlw   TMR0_INC        ; Reset timer increment
                addwf   TMR0,F 

                BANKSYM DIGIT
                incf    DIGIT,W         ; Bump digit counter by one
                movwf   DIGIT
                addlw   -3              ; Does counter exceed 2?
                btfsc   STATUS,C
                clrf    DIGIT           ; Yes, wrapped round

                movf    DIGIT,W         ; Fetch next digit number
                addlw   SEGMENTS        ; And calculate pattern address
                BANKIND SEGMENTS
                movwf   FSR             ; And prepare FSR

                call    GetControlBit   ; Get control mask

                BANKABS PORTA
                xorwf   PORTA,W         ; Figure out what has changed
                andlw   (1<<LED_D1)|(1<<LED_D2)|(1<<LED_D3)
                xorwf   PORTA,F         ; And change digit selections
                movf    INDF,W          ; Load  
                movwf   PORTB

                BANKSYM TICKS
                movf    TICKS,F         ; Has counter already reached zero?
                btfss   STATUS,Z        ; 
                incf    TICKS,F         ; No, ump general tick counter by one

                movf    PWM_TICKS,F     ; Has counter already reached zero?
                btfss   STATUS,Z
                incfsz  PWM_TICKS,F     ; No, ump PWM tick counter by one
                goto    Finished
                
                BANKABS PORTA
                bcf     PORTA,LED_PULSE ; Turn of LED
                BANKABS T2CON
                bcf     T2CON,TMR2ON    ; Turn off PWM generation
                BANKABS CCP1CON         
                clrf    CCP1CON         ; Configure PWM output
                BANKABS PORTB
                bcf     PORTB,3         ; And Force output lo

;------------------------------------------------------------------------------

Finished:
                movf    _PCLATH,W       ; Restore saved PCLATH
                movwf   PCLATH
                movf    _FSR,W          ; Restore saved FSR
                movwf   FSR                
                movf    _STATUS,W       ; Restore saved STATUS
                movwf   STATUS
                swapf   _WREG,F         ; Invert saved WREG
                swapf   _WREG,W         ; Restore original WREG
                retfie

; Returns a bit mask used to set the active digits anode to high.
;
; Result in WREG. Assumes bank selected. Uses 1 stack level.

GetControlBit:
                movlw   high AnodeBits  ; Prepare for computed jump
                movwf   PCLATH
                movf    DIGIT,W         ; Compute low address
                addlw   low AnodeBits
                btfsc   STATUS,C        ; Gone over a page boundary?
                incf    PCLATH,F        ; Yes, correct address
                movwf   PCL             ; Then jump into table
AnodeBits:
                retlw   (1<<LED_D1)
                retlw   (1<<LED_D2)
                retlw   (1<<LED_D3)

                end
