                nolist
;==============================================================================
; Optimised Bank Switching
;------------------------------------------------------------------------------

; Notes:
;
; This file defines a set of macros for each of the architectures that generate
; the additional instructions to switch the file banks for direct and indirect
; addressing.
;
; At the start of a module the active bank is initially unknown. The BANKSYM
; and BANKABS macros can be used to ensure that the correct bank is selected
; for direct access. BANKIND is used to prepare for an indirect access.
;
; The BANKANY macro is used to indicate a point in the code where the active
; bank has become unknown, for example at the start of a subroutine or after
; a CALL. The BANKABS macro attempts to generate the smallest number of
; instructions to change bank provided the current bank is known. If a BANKABS
; follows a BANKANY or BANKSYM a full set of instructions will be always be
; generated to ensure the correct bank is selected.
;
; Bank selection optimisation is primarily an issue for the 14-bit Mid-Range
; device.

;==============================================================================
; Revision History:
;
; 2006-01-21 AJ Initial version
;------------------------------------------------------------------------------

; The BANK_ symbol records either the address of a register in the current
; selected bank or -1 if the bank is not known.
 
BANK_           set     -1

; The BANKANY macro sets BANK_ to -1 to indicate that we don't know which
; bank is selected (e.g. when accessing a variable in a udata section).

BANKANY         macro
                nolist
BANK_           set     -1
                list
                endm

; The BANKSYM macro generates a full set of bank selection instructions to
; ensure the bank containing a relative symbol is correctly selected.

BANKSYM         macro   SYMBOL
                banksel SYMBOL
                BANKANY
                endm

; The BANKIND macro sets the FSR register to allow indirect access to the
; indicated symbol.

BANKIND         macro   SYMBOL
                bankisel SYMBOL
                endm

;==============================================================================
; 12-Bit Cores
;------------------------------------------------------------------------------

                if      HAS_12_BIT_CORE

; For 12-bit cores the upper bits of the FSR are used to select direct and
; indirect banks.

BANKABS         macro   FILE
                nolist
                 if      HAS_BANKED_RAM > 0
                  if      (BANK_ == -1) || ((FILE ^ BANK_) & h'20')
                   if      FILE & h'020'
                list
                bsf     FSR,5           ; Select bank 01
                nolist
                   else
                list
                bcf     FSR,5           ; Select bank 00
                nolist
                   endif
                  endif
                 endif
BANK_           set     FILE
                list
                endm
                
                endif

;==============================================================================
; 14-Bit Cores
;------------------------------------------------------------------------------

                if      HAS_14_BIT_CORE

; For 14-bit codes the RP0 and RP1 bits of the STATUS register are used to
; select the bank for direct addressing, while the IRP bit is used during
; indirect access.
;
; Not all 14-bit processors have multiple banks so the code must adject to
; 1, 2 and four bank configuration.

BANKABS         macro   FILE
                banksel FILE
                endm

BANKABS2         macro   FILE
                nolist
                 if      HAS_BANKED_RAM > 1
                  if      (BANK_ == -1) || ((FILE ^ BANK_) & h'100')
                   if      FILE & h'100'
                list
                bsf     STATUS,RP1      ; Set RP1
                nolist
                   else
                list
                bcf     STATUS,RP1      ; Clear RP1
                nolist
                   endif
                  endif
                 endif 
                 if      HAS_BANKED_RAM > 0
                  if      (BANK_ == -1) || ((FILE ^ BANK_) & h'080')
                   if      FILE & h'080'
                list
                bsf     STATUS,RP0      ; Set RP0
                nolist
                   else
                list
                bcf     STATUS,RP0      ; Clear RP0
                nolist
                   endif
                  endif
                 endif
BANK_           set     FILE
                list
                endm

                endif

;==============================================================================
; 16-Bit Cores
;------------------------------------------------------------------------------

                if      HAS_16_BIT_CORE

; The 16-bit core has a dedicated BSR to hold the bank for direct access
; operations and a special instruction (MOVLB) to change it. The BANKABS
; macro eliminates redundant selections of the same bank.

BANKABS         macro   FILE
                nolist
                 if      (BANK_ == -1) || ((FILE ^ BANK_) & h'f00')
                list
                banksel FILE
                nolist
                 endif
BANK_           set     FILE
                list
                endm

                endif

                list
